#
#
#  PythonXT Create Spots From File - Creates a spots component with spots positions, radii and timepoints from a simple text file
#  
#
#  Written by Sacha Guyer
#
#
#    <CustomTools>
#      <Menu>
#       <Item name="Create Spots From File" icon="Python" tooltip="Creating a spots component from a text file">
#         <Command>PythonXT::XTCreateSpotsFromFile(%i)</Command>
#       </Item>
#      </Menu>
#    </CustomTools>


import os
import ImarisLib
from Tkinter import Tk, Button, Label

def displayMessage(aMessage):
  """Shows the specified message in a Tkinter box with ok button"""
  vMaster = Tk()
  Label(vMaster, text=aMessage).pack()
  Button(vMaster, text='Ok', command=vMaster.destroy).pack()
  vMaster.mainloop()

def getSpotRadiusAndList(aFilename):
  """Parses the specified file and returns a dictionary with content to create a spots component"""
  vFile = open(aFilename)
  vLines = [vLine.strip() for vLine in vFile]

  # these are the default values
  vSpotRadius = 1
  vTimeIndex = 0

  # These values will be filled to the same size
  vSpotsPositions = []
  vTimeIndices = []
  vSpotsRadii = []

  # iterate through the lines
  for vLine in vLines:
    # catches much of the invalid values while parsing
    try:
      if 'SpotRadius' in vLine:
        vSpotRadius = float(vLine.split(',')[1].strip())
      elif 'TimeIndex' in vLine:
        vTimeIndex = int(vLine.split(',')[1].strip())
      else:
        vElements = [float(vElement) for vElement in vLine.split(',')]
        if len(vElements) == 3:
          vSpotsPositions.append(vElements)
          vSpotsRadii.append(vSpotRadius)
          vTimeIndices.append(vTimeIndex)
    except:
      pass
  vResult = {}
  vResult['SpotsPositions'] = vSpotsPositions
  vResult['TimeIndices'] = vTimeIndices
  vResult['TimeIndices'] = vTimeIndices
  vResult['SpotsRadii'] = vSpotsRadii
  return vResult
  
def XTCreateSpotsFromFile(aImarisId):

  # Create an ImarisLib object
  vImarisLib = ImarisLib.ImarisLib()
  
  # Get an imaris object with id aImarisId
  vImaris = vImarisLib.GetApplication(aImarisId)
  
  # Check if the object is valid
  if vImaris is None:
    vText = 'Could not connect to Imaris!'
    displayMessage(vText)
    return
    
  # Get the scene to later add the spots component
  vScene = vImaris.GetSurpassScene()
  if vScene is None:
    vText =  'Please add a scene to Imaris'
    displayMessage(vText)
    return

  # Get the currently loaded dataset  
  vImage = vImaris.GetDataSet()
  
  # This XTension requires a dataset
  if vImage is None:
    vText =  'An image has to be loaded into Imaris first'
    displayMessage(vText)
    return
  vImageNofTimePoints = vImage.GetSizeT()

  # get the spots details from the text file
  vFilename = 'XTCreateSpotsFromFile.txt'
  vSpotAndRadius = getSpotRadiusAndList(vFilename)
  vSpotsPositions = vSpotAndRadius['SpotsPositions']
  vTimeIndices = vSpotAndRadius['TimeIndices']
  vRadii = vSpotAndRadius['SpotsRadii']

  # remove items with invalid time index for the current dataset
  vItemsToPop = []
  for vIndex in range(len(vSpotsPositions)-1,0,-1):
    vTimePoint = vTimeIndices[vIndex]
    if vTimePoint < 0 or vTimePoint >= vImageNofTimePoints:
			vItemsToPop.append(vIndex)

  for vIndex in vItemsToPop:
      vSpotsPositions.pop(vIndex)
      vTimeIndices.pop(vIndex)
      vRadii.pop(vIndex)

  vNofToRemove = len(vItemsToPop)

  # Inform about spots that could not be added
  if (vNofToRemove > 0):
    vItemsToPop.sort()
    vText = str(vNofToRemove) + " points were removed because their timeindex did not fit, indices = " + str(vItemsToPop)
    displayMessage(vText)

  # Create a spots component
  vSpotsComponent = vImaris.GetFactory().CreateSpots();
  
  # Set the spots
  vSpotsComponent.Set(vSpotsPositions, vTimeIndices, vRadii);
  
  # Give the component a nice name
  vSpotsComponent.SetName('Spots from XTCreateSpotsFromFile');
  
  # Set red color
  vSpotsComponent.SetColorRGBA(255);

  # Add the spots component at the end of the surpass tree
  vScene.AddChild(vSpotsComponent, -1);
